!
SUBROUTINE rttov_tl( &
            & errorstatus,       &
            & chanprof,          &
            & opts,              &
            & profiles,          &
            & profiles_tl,       &
            & coefs,             &
            & transmission,      &
            & transmission_tl,   &
            & radiancedata,      &
            & radiancedata_tl,   &
            & calcemis,          &
            & emissivity,        &
            & emissivity_tl,     &
            & calcrefl,          &
            & reflectance,       &
            & reflectance_tl,    &
            & aer_opt_param,     &
            & cld_opt_param,     &
            & traj,              &
            & traj_tl,           &
            & pccomp,            &
            & pccomp_tl,         &
            & channels_rec)
!
! Description:
! Tangent Linear of rttov_direct
! to compute multi-channel level to space transmittances,
! top of atmosphere and level to space radiances and brightness
! temperatures and optionally surface emissivities, for many
! profiles in a single call, for satellite
! infrared or microwave sensors. The code requires a coefficient file
! for each sensor for which simulated radiances are requested.
!
! Copyright:
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2002, EUMETSAT, All Rights Reserved.
!
! Method: The methodology is described in the following:
!
! Eyre J.R. and H.M. Woolf  1988 Transmittance of atmospheric gases
! in the microwave region: a fast model. Applied Optics 27  3244-3249
!
! Eyre J.R. 1991 A fast radiative transfer model for satellite sounding
! systems.  ECMWF Research Dept. Tech. Memo. 176 (available from the
! librarian at ECMWF).
!
! Saunders R.W., M. Matricardi and P. Brunel 1999 An Improved Fast Radiative
! Transfer Model for Assimilation of Satellite Radiance Observations.
! QJRMS, 125, 1407-1425.
!
! Matricardi, M., F. Chevallier and S. Tjemkes 2001 An improved general
! fast radiative transfer model for the assimilation of radiance
! observations. ECMWF Research Dept. Tech. Memo. 345
! (available from the librarian at ECMWF).
!
! Matricardi, M. 2003 RTIASI-4, a new version of the ECMWF fast radiative
! transfer model for the infrared atmospheric sounding interferometer.
! ECMWF Research Dept. Tech. Memo. 425 (available from the librarian at ECMWF)
!
! Rochon Y.J., L. Garand, D.S. Turner and S. Polavarapu Jacobian mapping
! between  vertical coordinate systems in data assimilation (submitted QJRMS
! June 2006)
!
! Current Code Owner: SAF NWP
!
! History:
! Version   Date     Comment
! -------   ----     -------
!  1.1   01/12/2002  New F90 code with structures (P Brunel A Smith)
!  1.2   02/01/2003  More comments added (R Saunders)
!  1.3   24/01/2003  Error return code by input profile (P Brunel)
!  1.4               Add WV Continuum and CO2 capability
!  1.5   04/12/2003  Optimisation (J Hague and D Salmond ECMWF)
!  1.6   02/06/2004  Change tests on id_comp_lvl == 7 by tests on fmv_model_ver (P. Brunel)
!  1.7   08/12/2005  Added surface humidity to lowest mean layer q  (R Saunders)
!  1.8   23/01/2006  Marco Matricardi (ECMWF):
!              --    IASI capability added.
!              --    Variable trace gases added for IASI and AIRS.
!              --    Solar radiation added for IASI and AIRS
!              --    Altitude dependent local zenith angle added.
!              --    Linear in tau approximation for RT equation added.
!  1.9   23/01/2006 Changed intent of profiles_tl to inout (R Saunders)
!  1.10  31/03/2006  Initialised zlev (R Saunders)
!        06/02/2007  Removed polarisation indexing R. Saunders.
!        29/06/2007  Introduce interpolator (P.J.Rayer)
!              --    Profiles on USER levs -> COEF levs
!              --    Duplicate profaux and setgeometry (USER and COEF levs)
!              --    Predict atm/sol optical depths on COEF levs -> USER levs
!              --    Calculate transmittance and integrate RT on USER levs
!  1.11  11/10/2007  Move iaernum & iaertyp to profile_aux P.Marguinaud
!                    Initialize weights_calc array
!  1.12   16/01/2008 Facility to apply regression limits (N. Bormann)
!  1.13   26/09/2008 Disable vectorization of the pointer association
!                    loop and nullify temp pointers; this is for the NEC
!                    under -Chopt. P. Marguinaud
!  1.12   03/10/2008 Fix bug in checking addsolar
!  1.13   03/07/2009 Profile levels to include ToA. Distinguish between layer
!                    arrays  and level arrays - size, index labels, looping
!                    (P. Rayer)
!  1.14   03/11/2009 Transmittances / optical depths on levels (A Geer)
!  1.15   02/12/2009 Introduced principal component capability. Marco Matricardi. ECMWF
!  1.16   17/06/2010 Introduced optional spacetop flag to zero opdeps at user's
!                    model-top in Zeeman channels (P Rayer)
!  1.17   05/07/2010 Remove addsolar flag from profiles structure (J Hocking)
!  1.18    14/10/2010 Remove rt8_mode (J Hocking)
!
!
! 2010/03 Code cleaning Pascal Brunel, Philippe Marguinaud
!
!
! Code Description:
!   Language:           Fortran 90.
!   Software Standards: "European Standards for Writing and
!     Documenting Exchangeable Fortran 90 Code".
! A user guide and technical documentation is available at
! http://www.metoffice.com/research/interproj/nwpsaf/rtm/index.html
!
! Declarations:
! Modules used:
! Imported Parameters:
!INTF_OFF
#include "throw.h"
!INTF_ON
! Imported Type Definitions:
  USE rttov_types, ONLY :  &
       & rttov_options,     &
       & radiance_Type,     &
       & transmission_Type, &
       & profile_Type,      &
       & rttov_coefs,       &
       & rttov_pccomp,      &
       & rttov_chanprof,    &
       & rttov_emissivity,  &
       & rttov_reflectance, &
       & rttov_opt_param,   &
       & rttov_traj
  USE parkind1, ONLY : jpim, jplm
!INTF_OFF
  USE parkind1, ONLY : jprb
  USE rttov_const, ONLY :  &
         sensor_id_mw, &
         sensor_id_ir, &
         sensor_id_po, &
         sensor_id_hi, &
         ncldtyp
  USE yomhook, ONLY : LHOOK, DR_HOOK
  USE rttov_types, ONLY : &
       & rttov_traj_dyn,    &
       & rttov_traj_sta
!INTF_ON
  IMPLICIT NONE
!subroutine arguments:
  INTEGER(KIND=jpim)     , INTENT(OUT)                     :: errorstatus
  TYPE(profile_Type)     , INTENT(IN)                      :: profiles(:)
  TYPE(rttov_chanprof)   , INTENT(IN)                      :: chanprof(:)
  TYPE(rttov_options )   , INTENT(IN)                      :: opts
  TYPE(profile_Type  )   , INTENT(IN)                      :: profiles_tl(size(profiles))
  TYPE(rttov_coefs   )   , INTENT(IN)   , TARGET           :: coefs
  TYPE(transmission_Type), INTENT(INOUT)                   :: transmission     ! in because of mem allocation
  TYPE(transmission_Type), INTENT(INOUT)                   :: transmission_tl  ! in because of mem allocation
  TYPE(radiance_Type    ), INTENT(INOUT)                   :: radiancedata     ! in because of mem allocation
  TYPE(radiance_Type    ), INTENT(INOUT)                   :: radiancedata_tl  ! in because of mem allocation
  LOGICAL(KIND=jplm)     , INTENT(IN)   , OPTIONAL         :: calcemis(size(chanprof))
  TYPE(rttov_emissivity) , INTENT(INOUT), OPTIONAL         :: emissivity(size(chanprof))
  TYPE(rttov_emissivity) , INTENT(INOUT), OPTIONAL         :: emissivity_tl(size(chanprof))
  LOGICAL(KIND=jplm)     , INTENT(IN)   , OPTIONAL         :: calcrefl(size(chanprof))
  TYPE(rttov_reflectance), INTENT(INOUT), OPTIONAL         :: reflectance(size(chanprof))
  TYPE(rttov_reflectance), INTENT(INOUT), OPTIONAL         :: reflectance_tl(size(chanprof))
  TYPE(rttov_opt_param)  , INTENT(IN)   , OPTIONAL         :: aer_opt_param
  TYPE(rttov_opt_param)  , INTENT(IN)   , OPTIONAL         :: cld_opt_param
  TYPE(rttov_traj       ), INTENT(INOUT), OPTIONAL, TARGET :: traj, traj_tl    ! target is needed here
  TYPE(rttov_pccomp     ), INTENT(INOUT), OPTIONAL         :: pccomp
  TYPE(rttov_pccomp     ), INTENT(INOUT), OPTIONAL         :: pccomp_tl
  INTEGER(KIND=jpim)     , INTENT(IN)   , OPTIONAL         :: channels_rec(:)
!INTF_END
#include "rttov_errorreport.interface"
#include "rttov_apply_reg_limits_tl.interface"
#include "rttov_calcemis_ir_tl.interface"
#include "rttov_calcsurfrefl_tl.interface"
#include "rttov_profaux_tl.interface"
#include "rttov_setgeometry_tl.interface"
#include "rttov_setpredictors_7_tl.interface"
#include "rttov_setpredictors_8_tl.interface"
#include "rttov_setpredictors_9_tl.interface"
#include "rttov_opdep_tl.interface"
#include "rttov_opdep_9_tl.interface"
#include "rttov_transmit_tl.interface"
#include "rttov_transmit_9_solar_tl.interface"
#include "rttov_calcemis_mw_tl.interface"
#include "rttov_integrate_tl.interface"
#include "rttov_intavg_prof_tl.interface"
#include "rttov_intavg_chan_tl.interface"
#include "rttov_refsun_tl.interface"
#include "rttov_check_traj.interface"
#include "rttov_init_prof.interface"
#include "rttov_cldstr_tl.interface"
#include "rttov_opdpscattir_tl.interface"
#include "rttov_fresnel_tl.interface"
#include "rttov_init_raytracing.interface"
#include "rttov_pcscores_tl.interface"
#include "rttov_reconstruct_tl.interface"
#include "rttov_calcbt_pc_tl.interface"
#include "rttov_copy_raytracing.interface"
#include "rttov_copy_prof.interface"
#include "rttov_copy_aux_prof.interface"
#include "rttov_copy_opdp_path.interface"
#include "rttov_init_opdp_path.interface"
#include "rttov_alloc_traj_dyn.interface"
#include "rttov_alloc_traj_sta.interface"
#include "rttov_direct.interface"
#include "rttov_nlte_bias_correction_tl.interface"
#include "rttov_calcbt_tl.interface"
#include "rttov_calcsatrefl_tl.interface"

!local variables:
  INTEGER(KIND=jpim) :: nlevels
  LOGICAL(KIND=jplm) :: addcosmic! switch for adding temp of cosmic background

  TYPE(rttov_traj_dyn) :: traj0_dyn
  TYPE(rttov_traj_sta) :: traj0_sta
  TYPE(rttov_traj_dyn) :: traj0_tl_dyn

  LOGICAL(KIND=jplm)   :: ltraj_tl_dyn_dealloc

  TYPE(rttov_traj), TARGET  :: traj1
  TYPE(rttov_traj), POINTER :: traj0
  TYPE(rttov_traj), TARGET  :: traj1_tl
  TYPE(rttov_traj), POINTER :: traj0_tl
  INTEGER(KIND=jpim) :: nprofiles                    ! Number of profiles
  INTEGER(KIND=jpim) :: nchannels                    ! Number of radiances computed (channels used * profiles)
  INTEGER(KIND=jpim)  :: npcscores
  TYPE(rttov_options) :: opts_coef
  INTEGER(KIND=jpim)  :: ERR
  REAL   (KIND=JPRB)  :: ZHOOK_HANDLE

!- End of header --------------------------------------------------------
  TRY
!-------------
!0. initialize
!-------------
  IF (LHOOK) CALL DR_HOOK('RTTOV_TL', 0_jpim, ZHOOK_HANDLE)
  nprofiles                 = size(profiles)
  nchannels                 = size(chanprof)
  nlevels                   = profiles(1)%nlevels
  opts_coef                 = opts
  opts_coef%rt_ir%addclouds = .FALSE.
  opts_coef%rt_ir%addaerosl = .FALSE.
  errorstatus               = errorstatus_success

  ltraj_tl_dyn_dealloc = .FALSE.

  IF (opts%rt_ir%pc%addpc) THEN
    npcscores = size(pccomp%pcscores)
  ENDIF

  NULLIFY (traj0)
  NULLIFY (traj0_tl)
!-----------------------------------------------------------------------
!1. interpolator first call - input profiles from USER levs to COEF levs
!-----------------------------------------------------------------------
! done immediately in direct code - for quick escape if target values are out of bounds
  CALL rttov_check_traj( &
        & ERR,                 &
        & nprofiles,           &
        & nchannels,           &
        & opts,                &
        & nlevels,             &
        & coefs,               &
        & 1_jpim,              &
        & traj0 = traj0,       &
        & traj0_tl = traj0_tl, &
        & traj1 = traj1,       &
        & traj1_tl = traj1_tl, &
        & traj2 = traj,        &
        & traj2_tl = traj_tl)
  THROWM( ERR .NE. 0 , "rttov_check_traj fatal error")

  CALL rttov_direct( &
            & err,                           &
            & chanprof,                      &
            & opts,                          &
            & profiles,                      &
            & coefs,                         &
            & transmission,                  &
            & radiancedata,                  &
            & calcemis      = calcemis,      &
            & emissivity    = emissivity,    &
            & calcrefl      = calcrefl,      &
            & reflectance   = reflectance,   &
            & aer_opt_param = aer_opt_param, &
            & cld_opt_param = cld_opt_param, &
            & traj          = traj0,         &
            & traj_dyn      = traj0_dyn,     &
            & traj_sta      = traj0_sta,     &
            & pccomp        = pccomp,        &
            & channels_rec  = channels_rec)
  THROW(err.ne.0)

  IF (traj0_sta%dothermal .AND. .NOT. PRESENT(emissivity_tl)) THEN
    err = errorstatus_fatal
    THROWM(err.ne.0, "emissivity_tl parameter required")
  END IF
  IF (traj0_sta%dosolar .AND. .NOT. PRESENT(reflectance_tl)) THEN
    err = errorstatus_fatal
    THROWM(err.ne.0, "reflectance_tl parameter required")
  END IF

  CALL rttov_alloc_traj_dyn (err, traj0_tl_dyn, opts, nchannels, profiles(1)%nlayers, &
                             traj0_dyn%nstreams, ncldtyp, 1_jpim)
  THROW(err.ne.0)
  ltraj_tl_dyn_dealloc = .TRUE.
!
! Tangent Linear
!----------------
!
!---------------------------------------------------------------------------------------
!1. interpolator first call - input profiles from USER levs to COEF levs
!---------------------------------------------------------------------------------------
  CALL rttov_init_prof(traj0_tl%profiles_coef)
! intvar logical array set by direct code

  IF (opts%interpolation%addinterp) THEN
    CALL rttov_intavg_prof_tl( &
          & opts,                           &
          & nlevels,                        &
          & coefs%coef%nlevels,             &
          & profiles,                       &
          & profiles_tl,                    &
          & traj0%profiles_coef,            &
          & traj0_tl%profiles_coef,         &
          & coefs%coef,                     &
          & coefs%coef_pccomp)
  ELSE
    CALL rttov_copy_prof( &
          & traj0_tl%profiles_coef, &
          & profiles_tl,            &
          & larray = .TRUE._jplm,   &
          & lscalar = .FALSE._jplm)
  ENDIF

! complete profiles on COEF levs for surf, skin, cloud, aerosol, solar, angle
  CALL rttov_copy_prof( &
        & traj0_tl%profiles_coef, &
        & profiles_tl,            &
        & larray = .FALSE._jplm,  &
        & lscalar = .TRUE._jplm)
!------------------------------------------------------------------
!2. check input data is within suitable physical limits - COEF levs
!------------------------------------------------------------------

  IF (opts%config%apply_reg_limits .OR. opts%interpolation%reg_limit_extrap) THEN
    CALL rttov_apply_reg_limits_tl( &
          & opts,                        &
          & profiles,                    &
          & traj0_sta%profiles_coef_ref, &
          & traj0_tl%profiles_coef,      &
          & coefs%coef,                  &
          & coefs%coef_pccomp)
  ENDIF

!------------------------------------------------------------------------
!3. determine cloud top, surface levels, ice cloud parameters
!-----------------------------------------------------------------------
!3.2 COEF levs
!-------------
!   CALL rttov_init_aux_prof(traj0_tl%aux_prof_coef)
  CALL rttov_profaux_tl( &
    & opts_coef,              &
    & traj0%profiles_coef,    &
    & traj0_tl%profiles_coef, &
    & coefs%coef,             &
    & traj0%aux_prof_coef,    &
    & traj0_tl%aux_prof_coef)

!3.1 USER levs
!-------------
  IF (opts%interpolation%addinterp .or. opts%rt_ir%addclouds) THEN
   CALL rttov_profaux_tl( &
     & opts,              &
     & profiles,          &
     & profiles_tl,       &
     & coefs%coef,        &
     & traj0%aux_prof,    &
     & traj0_tl%aux_prof)
 ELSE
   CALL rttov_copy_aux_prof(traj0_tl%aux_prof, traj0_tl%aux_prof_coef)
 ENDIF

! TL on geometry
!------------------------------------------------------------------------
!4. set up common geometric variables for the RT integration
!------------------------------------------------------------------------
!4.2 COEF levs
!-------------
 CALL rttov_setgeometry_tl( &
   & opts,                     &
   & traj0_sta%dosolar,        &
   & traj0%profiles_coef,      &
   & traj0_tl%profiles_coef,   &
   & traj0%aux_prof_coef,      &
   & coefs%coef,               &
   & traj0_sta%angles_coef,    &
   & traj0%raytracing_coef,    &
   & traj0_tl%raytracing_coef)

!4.1 USER levs
!-------------
  CALL rttov_init_raytracing(traj0_sta%dosolar, traj0_tl%raytracing)
  IF (opts%interpolation%addinterp) THEN
    CALL rttov_setgeometry_tl( &
        & opts,                &
        & traj0_sta%dosolar,   &
        & profiles,            &
        & profiles_tl,         &
        & traj0%aux_prof,      &
        & coefs%coef,          &
        & traj0_sta%angles,    &
        & traj0%raytracing,    &
        & traj0_tl%raytracing)
  ELSE
    traj0_sta%angles = traj0_sta%angles_coef
    ! copy raytracing_coef *in to* raytracing_USER
    CALL rttov_copy_raytracing(traj0_sta%dosolar, traj0_tl%raytracing, traj0_tl%raytracing_coef)
  ENDIF

! TL of predictors
!---------------------------------------------------
!5. calculate atm/sol predictors - atm/sol COEF levs
!---------------------------------------------------

  IF (coefs%coef%fmv_model_ver == 7) THEN
    CALL rttov_setpredictors_7_tl( &
          & opts,                      &
          & traj0%profiles_coef,       &
          & traj0_tl%profiles_coef,    &
          & traj0_sta%angles_coef,     &
          & coefs%coef,                &
          & traj0%aux_prof_coef,       &
          & traj0_tl%aux_prof_coef,    &
          & traj0%predictors%path1,    &
          & traj0_tl%predictors%path1, &
          & traj0%raytracing_coef,     &
          & traj0_tl%raytracing_coef)
  ELSE IF (coefs%coef%fmv_model_ver == 8) THEN
    CALL rttov_setpredictors_8_tl( &
          & opts,                      &
          & traj0%profiles_coef,       &
          & traj0_tl%profiles_coef,    &
          & traj0_sta%angles_coef,     &
          & coefs%coef,                &
          & traj0%aux_prof_coef,       &
          & traj0%predictors%path1,    &
          & traj0_tl%predictors%path1, &
          & traj0%raytracing_coef,     &
          & traj0_tl%raytracing_coef)
  ELSE IF (coefs%coef%fmv_model_ver == 9) THEN
    IF (traj0_sta%dothermal) THEN
      CALL rttov_setpredictors_9_tl(            &
            & opts,                             &
            & traj0%profiles_coef,              &
            & traj0_tl%profiles_coef,           &
            & traj0_sta%angles_coef,            &
            & traj0%raytracing_coef%pathsat,    &
            & traj0_tl%raytracing_coef%pathsat, &
            & coefs%coef_pccomp,                &
            & coefs%coef,                       &
            & traj0%predictors%path1,           &
            & traj0_tl%predictors%path1)
    ENDIF
    IF (traj0_sta%dosolar) THEN
      CALL rttov_setpredictors_9_tl(            &
            & opts,                             &
            & traj0%profiles_coef,              &
            & traj0_tl%profiles_coef,           &
            & traj0_sta%angles_coef,            &
            & traj0%raytracing_coef%patheff,    &
            & traj0_tl%raytracing_coef%patheff, &
            & coefs%coef_pccomp,                &
            & coefs%coef,                       &
            & traj0%predictors%path2,           &
            & traj0_tl%predictors%path2)
    ENDIF
  ENDIF

!TL of optical depths
!--------------------------------------------------------------------
!6. predict atmospheric and solar optical depths - atm/sol COEF levs
!--------------------------------------------------------------------
  CALL rttov_init_opdp_path(opts, traj0_tl%opdp_path)
  CALL rttov_init_opdp_path(opts, traj0_tl%opdp_path_coef)

  IF (coefs%coef%fmv_model_ver == 9) THEN
    IF (traj0_sta%dothermal) THEN
      ! Calculate thermal path1 optical depths for all thermal channels
      CALL rttov_opdep_9_tl( &
            & coefs%coef%nlayers,                  &
            & chanprof,                            &
            & traj0_sta%thermal,                   &
            & traj0%predictors,                    &
            & traj0%predictors%path1,              &
            & traj0_tl%predictors%path1,           &
            & traj0%aux_prof_coef,                 &
            & traj0_tl%aux_prof_coef,              &
            & coefs%coef,                          &
            & coefs%coef%thermal,                  &
            & traj0%opdp_path_coef%atm_level,      &
            & traj0_tl%opdp_path_coef%atm_level,   &
            & traj0_sta%thermal_path1%opdp_ref_coef)
    ENDIF
    IF (traj0_sta%dosolar) THEN
      ! Calculate solar path2 optical depths for all solar channels
      CALL rttov_opdep_9_tl( &
            & coefs%coef%nlayers,                      &
            & chanprof,                                &
            & traj0_sta%solar,                         &
            & traj0%predictors,                        &
            & traj0%predictors%path2,                  &
            & traj0_tl%predictors%path2,               &
            & traj0%aux_prof_coef,                     &
            & traj0_tl%aux_prof_coef,                  &
            & coefs%coef,                              &
            & coefs%coef%solar,                        &
            & traj0%opdp_path_coef%sun_level_path2,    &
            & traj0_tl%opdp_path_coef%sun_level_path2, &
            & traj0_sta%solar_path2%opdp_ref_coef)
    ENDIF
  ELSE
    CALL rttov_opdep_tl( &
          & coefs%coef%nlayers,        &
          & chanprof,                  &
          & traj0%predictors,          &
          & traj0%predictors%path1,    &
          & traj0_tl%predictors%path1, &
          & traj0%aux_prof_coef,       &
          & traj0_tl%aux_prof_coef,    &
          & coefs%coef,                &
          & coefs%coef%thermal,        &
          & traj0%opdp_path_coef,      &
          & traj0_tl%opdp_path_coef,   &
          & traj0_sta%thermal_path1%opdp_ref_coef)
  ENDIF

!--------------------------------------------------------------------------
!7. interpolator second  call - optical depths from COEF levs to USER levs
!--------------------------------------------------------------------------

  IF (opts%interpolation%addinterp) THEN
    CALL rttov_intavg_chan_tl( &
          & opts,                           &
          & traj0_sta%thermal,              &
          & traj0_sta%solar,                &
          & coefs%coef%nlevels,             &
          & nlevels,                        &
          & chanprof,                       &
          & traj0%profiles_coef,            &
          & profiles,                       &
          & profiles_tl,                    &
          & traj0%opdp_path_coef,           &
          & traj0_tl%opdp_path_coef,        &
          & traj0%opdp_path,                &
          & traj0_tl%opdp_path)
  ELSE
    CALL rttov_copy_opdp_path(opts, traj0_tl%opdp_path, traj0_tl%opdp_path_coef)
  ENDIF

  IF (opts%interpolation%spacetop) THEN
    traj0_tl%opdp_path%atm_level(1,:) = 0._jprb
    IF (opts%rt_ir%addsolar) THEN
      traj0_tl%opdp_path%sun_level_path2(1,:) = 0._jprb
    ENDIF
  ENDIF

!! testop omitted
! TL of ircld
!--------------------------------------------------------------------
!8. If clouds are present,calculate the number of streams and
!   the cloud distribution  in each stream - remains on USER levs
!--------------------------------------------------------------------

  IF (opts%rt_ir%addclouds) THEN
    CALL rttov_cldstr_tl( &
          & opts%rt_ir,     &
          & profiles,       &
          & profiles_tl,    &
          & traj0%ircld,    &
          & traj0_tl%ircld)
  ELSE
    traj0_tl%ircld%xstrclr = 0._jprb
  ENDIF

! TL of opdpscattir
!----------------------------------------------------------------------------
!9. Calculate optical depths of aerosols and/or clouds - USER levs
!----------------------------------------------------------------------------

  IF (opts%rt_ir%addaerosl .OR. opts%rt_ir%addclouds) THEN
    CALL rttov_opdpscattir_tl( &
          & profiles(1)%nlayers,                           &
          & chanprof,                                      &
          & opts,                                          &
          & traj0%aux_prof,                                &
          & traj0_tl%aux_prof,                             &
          & profiles,                                      &
          & profiles_tl,                                   &
          & aer_opt_param,                                 &
          & cld_opt_param,                                 &
          & traj0_sta%dosolar,                             &
          & traj0_sta%solar,                               &
          & coefs%coef,                                    &
          & coefs%coef_scatt_ir,                           &
          & traj0%raytracing,                              &
          & traj0_tl%raytracing,                           &
          & traj0%transmission_scatt_ir,                   &
          & traj0_tl%transmission_scatt_ir,                &
          & traj0_dyn%transmission_scatt_ir_stream,        &
          & traj0_tl_dyn%transmission_scatt_ir_stream,     &
          & coefs%optp,                                    &
          & traj0%ircld,                                   &
          & traj0_tl%ircld)
  ENDIF

! TL of transmit
!----------------------------------------
!10. calculate transmittances - USER levs
!----------------------------------------

!TL of optical depths and transmittances
  IF (traj0_sta%dothermal) THEN
    CALL rttov_transmit_tl( &
          & opts%rt_ir%addaerosl,                           &
          & opts%rt_ir%addclouds,                           &
          & traj0_sta%do_lambertian,                        &
          & profiles(1)%nlayers,                            &
          & chanprof,                                       &
          & traj0_sta%thermal,                              &
          & traj0%aux_prof,                                 &
          & traj0_tl%aux_prof,                              &
          & coefs%coef,                                     &
          & traj0%ircld,                                    &
          & traj0_sta%angles,                               &
          & traj0%opdp_path%atm_level,                      &
          & traj0_tl%opdp_path%atm_level,                   &
          & traj0_sta%thermal_path1%od_level,               &
          & transmission%tau_levels,                        &
          & transmission%tau_total,                         &
          & transmission_tl%tau_levels,                     &
          & transmission_tl%tau_total,                      &
          & traj0_dyn%transmission_aux,                     &
          & traj0_dyn%transmission_aux%thermal_path1,       &
          & traj0_tl_dyn%transmission_aux%thermal_path1,    &
          & traj0%transmission_scatt_ir,                    &
          & traj0_tl%transmission_scatt_ir,                 &
          & traj0_dyn%transmission_scatt_ir_stream,         &
          & traj0_tl_dyn%transmission_scatt_ir_stream,      &
          & traj0_sta%thermal_path1%tau_ref,                &
          & traj0_sta%thermal_path1%tau_ref_surf,           &
          & traj0_sta%thermal_path1%tau_surf,               &
          & traj0_sta%thermal_path1%tau_level)
  ENDIF
  
  IF (traj0_sta%dosolar) THEN
    CALL rttov_transmit_9_solar_tl( &
          & opts%rt_ir%addaerosl,                          &
          & opts%rt_ir%addclouds,                          &
          & profiles(1)%nlayers,                           &
          & nprofiles,                                     &
          & chanprof,                                      &
          & traj0_sta%solar,                               &
          & traj0%aux_prof,                                &
          & traj0_tl%aux_prof,                             &
          & coefs%coef,                                    &
          & traj0%raytracing,                              &
          & traj0_tl%raytracing,                           &
          & traj0%ircld,                                   &
          & traj0%opdp_path,                               &
          & traj0_tl%opdp_path,                            &
          & traj0_sta%solar_path2,                         &
          & traj0_sta%solar_path1,                         &
          & transmission,                                  &
          & transmission_tl,                               &
          & traj0_dyn%transmission_aux,                    &
          & traj0_tl_dyn%transmission_aux,                 &
          & traj0%transmission_scatt_ir,                   &
          & traj0_tl%transmission_scatt_ir,                &
          & traj0_dyn%transmission_scatt_ir_stream,        &
          & traj0_tl_dyn%transmission_scatt_ir_stream)

  ENDIF

! TL of emissivity
!-------------------------------------------------
!11. calculate channel emissivity values - SURFACE
!-------------------------------------------------

  IF (traj0_sta%dothermal) THEN

    emissivity_tl%emis_out = emissivity_tl%emis_in
  
    traj0_tl%thermrefl =  - emissivity_tl%emis_out
  
    IF (ANY(calcemis) .OR. opts%rt_ir%pc%addpc) THEN
      ! Calculate surface emissivity and traj0%thermrefl for selected channels
  
      IF (coefs%coef%id_sensor == sensor_id_ir) THEN
        ! Infrared
        traj0_tl%thermrefl(:) =  - emissivity_tl(:)%emis_out
      ELSE IF (coefs%coef%id_sensor == sensor_id_mw .OR. coefs%coef%id_sensor == sensor_id_po) THEN
        ! Microwave
        CALL rttov_calcemis_mw_tl( &
              & opts,                             &
              & profiles,                         &
              & profiles_tl,                      &
              & traj0_sta%angles,                 &
              & coefs%coef,                       &
              & chanprof,                         &
              & traj0_dyn%transmission_aux,       &
              & traj0_tl_dyn%transmission_aux,    &
              & calcemis,                         &
              & emissivity_tl%emis_out,           &
              & traj0_tl%thermrefl)
      ELSE
        ! Hires
        CALL rttov_calcemis_ir_tl(    &
              & profiles,             &
              & profiles_tl,          &
              & coefs%coef,           &
              & opts%rt_ir%pc%addpc,  &
              & coefs%coef_pccomp,    &
              & traj0_sta%thermal,    &
              & chanprof,             &
              & calcemis,             &
              & emissivity_tl%emis_out)
        traj0_tl%thermrefl(:) =  - emissivity_tl(:)%emis_out
      ENDIF

    ENDIF

  ELSE IF (PRESENT(emissivity_tl)) THEN

    emissivity_tl%emis_out = 0._jprb

  ENDIF

! TL of reflectance
!-------------------------------------------------------
!12. Compute surface reflectance
!-------------------------------------------------------

  IF (traj0_sta%dosolar) THEN
  
    reflectance_tl%refl_out = reflectance_tl%refl_in
    
    IF (ANY(calcrefl)) THEN
    
      CALL rttov_refsun_tl( &
            & profiles,            &
            & profiles_tl,         &
            & coefs%coef,          &
            & traj0%aux_prof,      &
            & traj0%sunglint,      &
            & traj0_tl%sunglint,   &
            & traj0%raytracing,    &
            & traj0_tl%raytracing)
      CALL rttov_fresnel_tl( &
            & chanprof,           &
            & profiles,           &
            & traj0_sta%solar,    &
            & coefs%coef,         &
            & traj0%sunglint,     &
            & traj0_tl%sunglint,  &
            & traj0%fresnrefl,    &
            & traj0_tl%fresnrefl)

      CALL rttov_calcsurfrefl_tl(     &
            & coefs%coef,             &
            & profiles,               &
            & traj0%sunglint,         &
            & traj0_tl%sunglint,      &
            & traj0%fresnrefl,        &
            & traj0_tl%fresnrefl,     &
            & traj0_sta%solar,        &
            & chanprof,               &
            & traj0_sta%refl_norm,    &
            & calcrefl,               &
            & emissivity,             &
            & emissivity_tl,          &
            & reflectance_tl%refl_out)

    ENDIF

  ELSE IF (PRESENT(reflectance_tl)) THEN

    reflectance_tl%refl_out = 0._jprb

  ENDIF

! TL of RTE
!---------------------------------------------------------
!13. integrate the radiative transfer equation - USER levs
!---------------------------------------------------------

  addcosmic = (coefs%coef%id_sensor == sensor_id_mw .OR. coefs%coef%id_sensor == sensor_id_po)
  CALL rttov_integrate_tl( &
        & addcosmic,                                    &
        & opts,                                         &
        & traj0_dyn%nstreams,                           &
        & chanprof,                                     &
        & emissivity,                                   &
        & emissivity_tl,                                &
        & reflectance,                                  &
        & reflectance_tl,                               &
        & traj0_sta%refl_norm,                          &
        & traj0%thermrefl,                              &
        & traj0_tl%thermrefl,                           &
        & traj0_sta%do_lambertian,                      &
        & traj0_sta%thermal,                            &
        & traj0_sta%dothermal,                          &
        & traj0_sta%solar,                              &
        & traj0_sta%dosolar,                            &
        & traj0_sta%solar_spec_esd,                     &
        & traj0_dyn%transmission_aux,                   &
        & traj0_tl_dyn%transmission_aux,                &
        & traj0%transmission_scatt_ir,                  &
        & traj0_tl%transmission_scatt_ir,               &
        & profiles,                                     &
        & profiles_tl,                                  &
        & traj0%aux_prof,                               &
        & traj0_tl%aux_prof,                            &
        & coefs%coef,                                   &
        & traj0%raytracing,                             &
        & traj0_tl%raytracing,                          &
        & traj0%ircld,                                  &
        & traj0_tl%ircld,                               &
        & radiancedata,                                 &
        & traj0_sta%auxrad,                             &
        & traj0_dyn%auxrad_stream,                      &
        & traj0_tl_dyn%auxrad_stream,                   &
        & radiancedata_tl)


!---------------------------------------------------------
! 14. Calculate output BTs/reflectances/PCscores
!---------------------------------------------------------

  IF (opts%rt_ir%pc%addpc) THEN
    CALL rttov_pcscores_tl(          &
          & opts,                    &
          & chanprof,                &
          & traj0_sta%chanprof_pc,   &
          & pccomp,                  &
          & pccomp_tl,               &
          & coefs%coef_pccomp,       &
          & radiancedata_tl)

    IF (opts%rt_ir%pc%addradrec) THEN
      CALL rttov_reconstruct_tl(     &
            & opts,                  &
            & traj0_sta%chanprof_in, &
            & traj0_sta%chanprof_pc, &
            & pccomp,                &
            & pccomp_tl,             &
            & coefs%coef_pccomp)
    ENDIF
  ENDIF

! Do NLTE bias correction for hyperspectral instruments (not PC-RTTOV)
  IF (.NOT. opts%rt_ir%pc%addpc) THEN
    IF (coefs%coef%nltecoef) THEN
      IF (opts%rt_ir%do_nlte_correction .AND. &
          coefs%coef%id_sensor == sensor_id_hi) THEN
        CALL rttov_nlte_bias_correction_tl(coefs%coef, profiles, profiles_tl, &
                                           traj0_sta%angles, chanprof, radiancedata_tl)
      ENDIF
    ENDIF
  ENDIF

  IF (opts%rt_ir%pc%addpc) THEN

    IF (opts%rt_ir%pc%addradrec) &
      CALL rttov_calcbt_pc_tl(&
           traj0_sta%chanprof_in, &
           coefs%coef_pccomp,     &
           pccomp,                &
           pccomp_tl)

  ELSE

    IF (traj0_sta%dothermal) THEN
      CALL rttov_calcbt_tl(chanprof, coefs%coef, traj0_sta%thermal, radiancedata, radiancedata_tl)
    ELSE
      radiancedata_tl%bt_clear(:) = 0._jprb
      radiancedata_tl%bt(:) = 0._jprb
    ENDIF
    IF (traj0_sta%dosolar) THEN
      CALL rttov_calcsatrefl_tl(chanprof, profiles, traj0_sta%solar_spec_esd, traj0_sta%solar, radiancedata_tl)
    ELSE
      radiancedata_tl%refl_clear(:) = 0._jprb
      radiancedata_tl%refl(:) = 0._jprb
    ENDIF
  ENDIF

!--------------------
!15. deallocate memory
!--------------------

  CALL cleanup()

  IF (LHOOK) CALL DR_HOOK('RTTOV_TL', 1_jpim, ZHOOK_HANDLE)
  CATCH_C
  errorstatus = ERR
  IF (LHOOK) CALL DR_HOOK('RTTOV_TL', 1_jpim, ZHOOK_HANDLE)

CONTAINS

  SUBROUTINE cleanup()
    INTEGER(KIND=jpim) :: error

    IF ( ASSOCIATED(traj0_dyn%transmission_aux%fac1) ) THEN
      CALL rttov_alloc_traj_dyn (error, traj0_dyn, opts, nchannels, profiles(1)%nlayers, &
                                 traj0_dyn%nstreams, ncldtyp, 0_jpim)
    ENDIF

    IF ( ltraj_tl_dyn_dealloc ) THEN
      CALL rttov_alloc_traj_dyn (error, traj0_tl_dyn, opts, nchannels, profiles(1)%nlayers, &
                                 traj0_dyn%nstreams, ncldtyp, 0_jpim)
    ENDIF

    IF ( ASSOCIATED(traj0_sta%thermal) ) THEN
      CALL rttov_alloc_traj_sta (error, traj0_sta, opts, coefs%coef, nlevels, nchannels, nprofiles, &
                                 0_jpim, npcscores, channels_rec)
    ENDIF

    IF ( ASSOCIATED(traj0) ) THEN
      CALL rttov_check_traj( &
            & error,               &
            & nprofiles,           &
            & nchannels,           &
            & opts,                &
            & nlevels,             &
            & coefs,               &
            & 0_jpim,              &
            & traj0 = traj0,       &
            & traj0_tl = traj0_tl, &
            & traj1 = traj1,       &
            & traj1_tl = traj1_tl, &
            & traj2 = traj,        &
            & traj2_tl = traj_tl)
    ENDIF
  END SUBROUTINE cleanup

END SUBROUTINE rttov_tl
