  !
  ! Description:
  !    Parallel ( openmp ) version of RTTOV_DIRECT, _TL, _AD and _K.   
  !    This file contains the definition of RTTOV_PARALLEL_DIRECT, _TL, _AD and _K. 
  !    RTTOV_PARALLEL_DIRECT is activated when _RTTOV_PARALLEL_DIRECT is defined, etc...
  !    The interfaces of these parallel routines are identical to the original ones,
  !    plus the optional parameter nthreads.
  !
  ! Copyright:
  !
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2007, EUMETSAT, All Rights Reserved.
  !
  !     *************************************************************
  !
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !          09/2007    Creation P. Marguinaud & P. Brunel
  !          10/2008    Fix for po sounders P. Marguinaud
  !          11/2009    Add strategy and debug parameters P. Marguinaud
  !


Subroutine rttov_parallel_ad      &
      & ( errorstatus      &
      & , chanprof         &
      & , opts             &
      & , profiles         &
      & , profiles_ad      &
      & , coefs            &
      & , transmission     &
      & , transmission_ad  &
      & , radiancedata     &
      & , radiancedata_ad  &
      & , calcemis         &
      & , emissivity       &
      & , emissivity_ad    &
      & , calcrefl         &
      & , reflectance      &
      & , reflectance_ad   &
      & , aer_opt_param    &
      & , cld_opt_param    &
      & , traj             &
      & , traj_ad          &
      &,  pccomp           &
      &,  pccomp_ad        &
      &,  channels_rec     &
      &,  nthreads         &
      &,  strategy         &
      &,  debug            &
& )

!INTF_OFF
  Use rttov_const, Only :  &
       & errorstatus_success,&
       & errorstatus_fatal,  &  
       & sensor_id_po
!INTF_ON
  Use rttov_types, Only :            &
        & rttov_options,             &
        & rttov_chanprof,            &
        & rttov_emissivity,          &
        & rttov_reflectance,         &
        & rttov_opt_param,           &
        & rttov_pccomp,              &
        & rttov_coefs,               &
        & profile_Type,              &
        & transmission_type,         &
        & radiance_Type,             &
        & rttov_traj

  Use parkind1, Only : jpim, jplm
  Implicit None



!subroutine arguments:
  Integer(Kind=jpim),             Intent(out)   :: errorstatus ! return flag
  Type(rttov_chanprof),           Intent(in)    :: chanprof(:)
  Type(rttov_options),            Intent(in)    :: opts
  Type(profile_Type),             Intent(in)    :: profiles( : ) ! Atmospheric profiles
  Type(rttov_coefs),              Intent(in)    :: coefs
  Type(transmission_type),        Intent(inout) :: transmission     ! transmittances and layer optical depths
  Type(radiance_Type),            Intent(inout) :: radiancedata     ! radiances (mw/cm-1/ster/sq.m) and degK
  Type(profile_Type),             Intent(inout) :: profiles_ad( : ) 
  Type(transmission_type),        Intent(inout) :: transmission_ad
  Type(radiance_Type),            Intent(inout) :: radiancedata_ad 

  Logical(Kind=jplm),      Optional, Target, Intent(in)    :: calcemis( : )    ! switch for emissivity calc.
  Type(rttov_emissivity),  Optional, Target, Intent(inout) :: emissivity( : )  ! surface emissivity
  Type(rttov_emissivity),  Optional, Target, Intent(inout) :: emissivity_ad( : )
  
  Logical(Kind=jplm),      Optional, Target, Intent(in)    :: calcrefl( : )    ! switch for reflectance calc.
  Type(rttov_reflectance), Optional, Target, Intent(inout) :: reflectance( : ) ! surface reflectance
  Type(rttov_reflectance), Optional, Target, Intent(inout) :: reflectance_ad( : )

  Type(rttov_opt_param), Optional, Intent(in)  :: aer_opt_param
  Type(rttov_opt_param), Optional, Intent(in)  :: cld_opt_param

  Type(rttov_traj),    Optional, Intent(inout) :: traj
  Type(rttov_traj),    Optional, Intent(inout) :: traj_ad
  Type(rttov_pccomp),  Optional, Intent(inout) :: pccomp
  Type(rttov_pccomp),  Optional, Intent(inout) :: pccomp_ad
  Integer(Kind=jpim),  Optional, Intent(in)    :: channels_rec( : )
  

  Integer(Kind=jpim),  Optional, Intent(in)    :: nthreads
  ! Strategy: 0 = no strategy (RTTOV computes band limits), 1 = profile-wise, 2 = channel-wise
  Integer(Kind=jpim),  Optional, Intent(in)    :: strategy
  Logical(Kind=jplm),  Optional, Intent(in)    :: debug

!INTF_END

#include "rttov_ad.interface"
#include "rttov_alloc_prof.interface"
#include "rttov_init_prof.interface"
#include "rttov_add_prof.interface"

#include "rttov_errorreport.interface"




  Character(len=*), Parameter :: NameOfRoutine = &
    "rttov_parallel_ad"



  Integer(Kind=jpim) :: nprofiles12max
  Integer(Kind=jpim) :: k
  Integer(Kind=jpim) :: iband, nbands
  Integer(Kind=jpim) :: mthreads
  Integer(Kind=jpim) :: nsplits
  Integer(Kind=jpim) :: ichannelk, iprofilek
  Integer(Kind=jpim) :: nlevels
  Integer(Kind=jpim) :: nchannels
  Integer(Kind=jpim) :: nprofiles
  Integer(Kind=jpim) :: strategy1
  Logical(Kind=jplm) :: debug1
  Integer            :: mstat
  Integer            :: npcscores, nchan_rec
  Integer(Kind=jpim),      Allocatable :: errorstatus_x( : )               ! errorstatus/bands
  Integer(Kind=jpim),      Allocatable :: ichannel1( : ), ichannel2( : )   ! channel limits
  Integer(Kind=jpim),      Allocatable :: iprofile1( : ), iprofile2( : )   ! profile limits
  Integer(Kind=jpim),      Allocatable :: ipc1( : ), ipc2( : )             ! PCscores limits
  Integer(Kind=jpim),      Allocatable :: ichannelrec1( : ), ichannelrec2( : ) ! PC reconstructed channel limits
  Type(rttov_chanprof),    Allocatable :: chanprof_x( :, : )               ! chanprof/bands
  Type(rttov_opt_param),   Allocatable :: aer_opt_param_x( : )             ! user aerosol opt params
  Type(rttov_opt_param),   Allocatable :: cld_opt_param_x( : )             ! user cloud opt params

  Logical(Kind=jplm),      Pointer :: calcemis_x( : )             ! switch for emis/bands ( pointer assoc )
  Type(rttov_emissivity),  Pointer :: emissivity_x( : )           ! surface emis/bands ( pointer assoc )
  Type(rttov_emissivity),  Pointer :: emissivity_ad_x( : )
  
  Logical(Kind=jplm),      Pointer :: calcrefl_x( : )             ! switch for refl/bands ( pointer assoc )
  Type(rttov_reflectance), Pointer :: reflectance_x( : )          ! surface refl/bands ( pointer assoc )
  Type(rttov_reflectance), Pointer :: reflectance_ad_x( : )

  Type(transmission_type), Allocatable :: transmission_x( : )         ! transmission/bands ( pointer assoc )
  Type(radiance_Type),     Allocatable :: radiancedata_x( : )         ! radiancedata/bands ( pointer assoc )
  Type(rttov_pccomp),      Allocatable :: pccomp_x( : )               ! pccomp/bands ( pointer assoc )

  Integer(Kind=jpim)                   :: errorstatus_ad_x
  Type(transmission_type), Allocatable :: transmission_ad_x( : )  
  Type(radiance_Type),     Allocatable :: radiancedata_ad_x( : )
  Type(rttov_pccomp),      Allocatable :: pccomp_ad_x( : )
  Type(profile_Type),      Allocatable :: profiles_ad_x( :, : ) 


! Activated if openmp
!  
!$  Integer, External :: omp_get_thread_num
!$  Integer, External :: omp_get_max_threads
!

  nchannels = size(chanprof)
  nprofiles = size(profiles)

  debug1 = .FALSE.
  If( Present( debug ) ) debug1 = debug

  If( Present( traj ) .AND. opts%config%verbose ) Then
    Call rttov_errorreport( errorstatus_success, "traj argument will not be used !!", NameOfRoutine )
  EndIf
  If( Present( traj_ad ) .AND. opts%config%verbose ) Then
    Call rttov_errorreport( errorstatus_success, "traj_ad argument will not be used !!", NameOfRoutine )
  EndIf


  nlevels = profiles( 1 ) % nlevels

  strategy1 = 0
  If( Present( strategy ) ) Then
    strategy1 = strategy
  EndIf
  ! Must distribute work profile-wise among threads for PC-RTTOV and polarised sensors
  If ( opts%rt_ir%pc%addpc .or. coefs%coef%id_sensor .eq. sensor_id_po ) strategy1 = 1

  If( Present( nthreads ) ) Then
    mthreads = nthreads
  Else
! Default value for mthreads
    mthreads = 1_jpim
! Activated if openmp
!  
!$  mthreads = omp_get_max_threads()
!
  EndIf

! Check we do not have more bands than channels

  nsplits = 1_jpim
  nbands = mthreads * nsplits


  If( strategy1 .eq. 0 ) Then
    If( nbands .gt. nchannels ) nbands = nchannels
  Else If( strategy1 .eq. 1 ) Then
    nbands = nprofiles
  Else If( strategy1 .eq. 2 ) Then
    nbands = nchannels
  Else
    errorstatus = errorstatus_fatal
    Call rttov_errorreport( errorstatus_fatal, "Unknown strategy", NameOfRoutine )
    Return
  Endif
  

  If( debug1 ) Then
  Write( *, '(A20," = ",I8)' ) "nprofiles", nprofiles
  Write( *, '(A20," = ",I8)' ) "nchannels", nchannels
  Write( *, '(A20," = ",I5)' ) "nbands", nbands
  Endif
  
  Allocate( ichannel1( nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
  Allocate( ichannel2( nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
  
  If( opts%rt_ir%pc%addpc ) Then
    Allocate( ipc1( nbands ), stat = mstat )
    If( mstat .ne. 0 ) Goto 100
    Allocate( ipc2( nbands ), stat = mstat )
    If( mstat .ne. 0 ) Goto 100
    If( opts%rt_ir%pc%addradrec ) Then
      Allocate( ichannelrec1( nbands ), stat = mstat )
      If( mstat .ne. 0 ) Goto 100
      Allocate( ichannelrec2( nbands ), stat = mstat )
      If( mstat .ne. 0 ) Goto 100
    EndIf
  EndIf
  
  Allocate( iprofile1( nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
  Allocate( iprofile2( nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
  
  Allocate( transmission_x( nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
  Allocate( radiancedata_x( nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
  If( opts%rt_ir%pc%addpc ) Then
    Allocate( pccomp_x( nbands ), stat = mstat )
    If( mstat .ne. 0 ) Goto 100
  EndIf

  Allocate( aer_opt_param_x( nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
  Allocate( cld_opt_param_x( nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100

  Allocate( transmission_ad_x( nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
  Allocate( radiancedata_ad_x( nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
  If( opts%rt_ir%pc%addpc ) Then
    Allocate( pccomp_ad_x( nbands ), stat = mstat )
    If( mstat .ne. 0 ) Goto 100
  EndIf
  
  If ( strategy1 .eq. 1 ) Then
    If ( opts%rt_ir%pc%addpc ) Then
      ! Always the same number of channels per profile for PCs (nbands == nprofiles)
      Allocate( chanprof_x( nchannels / nbands, nbands ), stat = mstat )
    Else If ( coefs%coef%id_sensor .eq. sensor_id_po ) Then
      ! Account for extreme variations in channels per profile
      Allocate( chanprof_x( coefs%coef%fmv_chn, nbands ), stat = mstat )
    Else
      ! In the general case ensure chanprof_x is large enough if one profile
      ! is run for every channel and another is run for only one channel.
      ! The only way to do this without explicitly finding the maximum number of
      ! channels per profile is to size chanprof_x by chanprof. This shouldn't be a
      ! problem since the prof-by-prof strategy is intended for debugging rather than
      ! for general use.
      Allocate( chanprof_x( nchannels, nbands ), stat = mstat )
    EndIf
  Else If ( strategy1 .eq. 2 ) Then
    ! One channel per band (i.e. nbands == nchannels)
    Allocate( chanprof_x( 1, nbands ), stat = mstat )
  Else
    Allocate( chanprof_x( nchannels / nbands + nbands, nbands ), stat = mstat )
  EndIf
  If( mstat .ne. 0 ) Goto 100
  Allocate( errorstatus_x( nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100

  ichannel1( : ) = 0_jpim
  ichannel2( : ) = 0_jpim
  
  If( opts%rt_ir%pc%addpc ) Then
    ipc1( : ) = 0_jpim
    ipc2( : ) = 0_jpim
    If( opts%rt_ir%pc%addradrec ) Then
      ichannelrec1( : ) = 0_jpim
      ichannelrec2( : ) = 0_jpim
    EndIf
  EndIf
  
  iprofile1( : ) = 0_jpim
  iprofile2( : ) = 0_jpim
  
  !
  ! Build the array of lower/upper bounds of bands
  !

  If( strategy1 .eq. 1 ) Then
    If( opts%rt_ir%pc%addpc ) Then
      npcscores = size(pccomp%pcscores) / size(profiles)
      If( opts%rt_ir%pc%addradrec ) nchan_rec = size(channels_rec)
    EndIf
    ichannelk = 1_jpim
    Do iband = 1, nbands
      ichannel1( iband ) = ichannelk
      iprofilek = chanprof(ichannelk)%prof
      Do
        If( ichannelk .gt. nchannels ) Exit
        If( iprofilek .gt. nprofiles ) Exit
        If( iprofilek .ne. chanprof(ichannelk)%prof ) Exit
        ichannelk = ichannelk + 1_jpim
      EndDo
      ichannel2( iband ) = ichannelk-1_jpim
      If( opts%rt_ir%pc%addpc ) Then
        ! With one profile per band, splitting the pcscores and reconstructed channels
        ! among bands is quite simple
        ipc1( iband ) = (iband-1_jpim) * npcscores + 1_jpim
        ipc2( iband ) = ipc1( iband ) + npcscores - 1_jpim
        If( opts%rt_ir%pc%addradrec ) Then
          ichannelrec1( iband ) = (iband-1_jpim) * nchan_rec + 1_jpim
          ichannelrec2( iband ) = ichannelrec1( iband ) + nchan_rec - 1_jpim
        EndIf
      EndIf
    EndDo
  Else
    !
    ! For IR & MW sensors, we try to make bands
    ! with the same number of channels
    !
    ichannelk = 1_jpim
    Do iband = 1, nbands
      ichannel1( iband ) = ichannelk
      ichannelk = ichannelk + nchannels / nbands - 1_jpim
      If( iband .eq. nbands ) ichannelk = nchannels
      ichannel2( iband ) = ichannelk
      ichannelk = ichannelk + 1_jpim
    End Do
  End If

  chanprof_x( :, : )%chan = 0_jpim
  chanprof_x( :, : )%prof = 0_jpim

  Do iband = 1, nbands
    iprofile1( iband ) = chanprof( ichannel1( iband ) )%prof
    iprofile2( iband ) = chanprof( ichannel2( iband ) )%prof
    chanprof_x( 1 : ichannel2( iband ) - ichannel1( iband ) + 1, iband )%chan = &
          & chanprof( ichannel1( iband ) : ichannel2( iband ) )%chan
    Do ichannelk = ichannel1( iband ), ichannel2( iband )
      chanprof_x( ichannelk - ichannel1( iband ) + 1, iband )%prof &
        & = chanprof( ichannelk )%prof - iprofile1( iband ) + 1
    EndDo
  EndDo 
  
  !
  ! for rttov_ad, we have to allocate a temporary array holding
  ! results from every band, and add the contribution of every band
  ! after the parallel loop
  !
  nprofiles12max = maxval( iprofile2( 1 : nbands ) - iprofile1( 1 : nbands ) + 1 )
  Allocate( profiles_ad_x( nprofiles12max, nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
  Do iband = 1, nbands
    call rttov_alloc_prof( errorstatus_ad_x, nprofiles12max, profiles_ad_x( :, iband ), nlevels, opts, 1_jpim, & 
      init = .true._jplm, coefs=coefs )
    If( errorstatus_ad_x .ne. 0 ) Goto 100
    Call rttov_init_prof(profiles_ad_x( :, iband ))
  EndDo
  
  
  If( debug1 ) Then

  Do iband = 1, nbands
    Write( *, * )
    Write( *, '(A20," = ",I5)' ) "band", iband
    Write( *, '(A20," = ",100(I5))' ) "channels", chanprof( ichannel1( iband ) : ichannel2( iband ) )%chan
    Write( *, '(A20," = ",100(I5))' ) "lprofiles", chanprof( ichannel1( iband ) : ichannel2( iband ) )%prof
    Write( *, '(A20," = ",100(I5))' ) "chanprof_x%chan", chanprof_x( :, iband )%chan
    Write( *, '(A20," = ",100(I5))' ) "chanprof_x%prof", chanprof_x( :, iband )%prof
    Write( *, * )
  EndDo

  Endif
  
  If (Present(calcemis)) Then
    calcemis_x => calcemis
  Else
    Allocate( calcemis_x( nchannels ), stat = mstat )
    If( mstat .ne. 0 ) Goto 100
    calcemis_x(:) = .true.
  EndIf
  
  If (Present(emissivity)) Then
    emissivity_x => emissivity
  Else
    Allocate( emissivity_x( nchannels ), stat = mstat )
    If( mstat .ne. 0 ) Goto 100
  EndIf
  
  If (Present(emissivity_ad)) Then
    emissivity_ad_x => emissivity_ad
  Else
    Allocate( emissivity_ad_x( nchannels ), stat = mstat )
    If( mstat .ne. 0 ) Goto 100
    emissivity_ad_x(:) % emis_in  = 0.
    emissivity_ad_x(:) % emis_out = 0.
  EndIf

  If (Present(calcrefl)) Then
    calcrefl_x => calcrefl
  Else
    Allocate( calcrefl_x( nchannels ), stat = mstat )
    If( mstat .ne. 0 ) Goto 100
    calcrefl_x(:) = .true.
  EndIf

  If (Present(reflectance)) Then
    reflectance_x => reflectance
  Else
    Allocate( reflectance_x( nchannels ), stat = mstat )
    If( mstat .ne. 0 ) Goto 100    
  EndIf
  
  If (Present(reflectance_ad)) Then
    reflectance_ad_x => reflectance_ad
  Else
    Allocate( reflectance_ad_x( nchannels ), stat = mstat )
    If( mstat .ne. 0 ) Goto 100
    reflectance_ad_x(:) % refl_in  = 0.
    reflectance_ad_x(:) % refl_out = 0.
  EndIf


  !
  ! we make the correct pointers associations between temporary transmissions
  ! and radiances and arrays passed as input
  !
  Do iband = 1, nbands
  
    Call AssociateTransmission( ichannel1( iband ), ichannel2( iband ), &
     & transmission_x( iband ), transmission )
    Call AssociateRadiance( ichannel1( iband ), ichannel2( iband ), &
     & radiancedata_x( iband ), radiancedata )

    If( opts%rt_ir%pc%addpc ) Then
      If( opts%rt_ir%pc%addradrec ) Then
        Call AssociatePCcomp( ipc1( iband ), ipc2( iband ), &
        & pccomp_x( iband ), pccomp, ichannelrec1( iband ), ichannelrec2( iband ) )
      Else
        Call AssociatePCcomp( ipc1( iband ), ipc2( iband ), &
        & pccomp_x( iband ), pccomp )
      EndIf
    EndIf

    If( Present(aer_opt_param) .AND. opts%rt_ir%user_aer_opt_param ) Then
      Call AssociateOptParam( ichannel1( iband ), ichannel2( iband ), &
      & aer_opt_param_x( iband ), aer_opt_param )
    EndIf
    If( Present(cld_opt_param) .AND. opts%rt_ir%user_cld_opt_param ) Then
      Call AssociateOptParam( ichannel1( iband ), ichannel2( iband ), &
      & cld_opt_param_x( iband ), cld_opt_param )
    EndIf

    Call AssociateTransmission( ichannel1( iband ), ichannel2( iband ), &
     & transmission_ad_x( iband ), transmission_ad )
    Call AssociateRadiance( ichannel1( iband ), ichannel2( iband ), &
     & radiancedata_ad_x( iband ), radiancedata_ad )
     
    If( opts%rt_ir%pc%addpc ) Then
      If( opts%rt_ir%pc%addradrec ) Then
        Call AssociatePCcomp( ipc1( iband ), ipc2( iband ), &
        & pccomp_ad_x( iband ), pccomp_ad, ichannelrec1( iband ), ichannelrec2( iband ) )
      Else
        Call AssociatePCcomp( ipc1( iband ), ipc2( iband ), &
        & pccomp_ad_x( iband ), pccomp_ad )
      EndIf
    EndIf 
    
    
  End Do

  
  
  
!$OMP PARALLEL DO PRIVATE( iband ) NUM_THREADS( mthreads ) SCHEDULE( DYNAMIC )
Do iband = 1, nbands

  If( debug1 ) Then
!    Print *, "================================================"
!    Print *, "THREAD = ", OMP_GET_THREAD_NUM(), " BAND = ", iband, &
!  & " ichannel1 = ", ichannel1( iband ), " ichannel2 = ", ichannel2( iband )

    Write( *, * )
    Write( *, '(A)' ) "=========================== rttov_ad ==========================="
    Write( *, '(A20)' ) "errorstatus"
    Write( *, '(A20," = ",I5)' ) "nprofiles", iprofile2( iband ) - iprofile1( iband ) + 1
    Write( *, '(A20," = ",I5)' ) "nchannels", ichannel2( iband ) - ichannel1( iband ) + 1
    Write( *, '(A20," = ",100(I5))' ) "channels", chanprof_x( :, iband )%chan
    Write( *, '(A20," = ",100(I5))' ) "lprofiles", chanprof_x( :, iband )%prof
    Write( *, '(A20," = ",A20," ( ",I5," : ",I5," ) ")' ) "profiles",          &
    &"profiles", iprofile1( iband ), iprofile2( iband )
    Write( *, '(A20," = ",A20," ( ",I5," : ",I5," ) ")' ) "profiles_ad",       &
    &"profiles_ad", iprofile1( iband ), iprofile2( iband )
    Write( *, '(A20)' ) "coefs"

#ifndef RTTOV_NAG53
    If (Present(calcemis)) Then
      Write( *, '(A20," = ",A20," ( ",I5," : ",I5," ) ")' ) "calcemis",        &
      &"calcemis", ichannel1( iband ), ichannel2( iband )
    Endif
    If (Present(emissivity)) Then
      Write( *, '(A20," = ",A20," ( ",I5," : ",I5," ) ")' ) "emis%emis_in",    &
      &"emis%emis_in", ichannel1( iband ), ichannel2( iband )
      Write( *, '(A20," = ",A20," ( ",I5," : ",I5," ) ")' ) "emis%emis_out",   &
      &"emis%emis_out", ichannel1( iband ), ichannel2( iband )
    Endif
    If (Present(emissivity_ad)) Then
      Write( *, '(A20," = ",A20," ( ",I5," : ",I5," ) ")' ) "emis_ad%emis_in", &
      &"emis_ad%emis_in", ichannel1( iband ), ichannel2( iband )
      Write( *, '(A20," = ",A20," ( ",I5," : ",I5," ) ")' ) "emis_ad%emis_out", &
      &"emis_ad%emis_out", ichannel1( iband ), ichannel2( iband )
    Endif
    If (Present(calcrefl)) Then
      Write( *, '(A20," = ",A20," ( ",I5," : ",I5," ) ")' ) "calcrefl",        &
      &"calcrefl", ichannel1( iband ), ichannel2( iband )
    Endif
    If (Present(reflectance)) Then
      Write( *, '(A20," = ",A20," ( ",I5," : ",I5," ) ")' ) "refl%refl_in",    &
      &"refl%refl_in", ichannel1( iband ), ichannel2( iband )
      Write( *, '(A20," = ",A20," ( ",I5," : ",I5," ) ")' ) "refl%refl_out",   &
      &"refl%refl_out", ichannel1( iband ), ichannel2( iband )
    Endif
    If (Present(reflectance_ad)) Then
      Write( *, '(A20," = ",A20," ( ",I5," : ",I5," ) ")' ) "refl_ad%refl_in", &
      &"refl_ad%refl_in", ichannel1( iband ), ichannel2( iband )
      Write( *, '(A20," = ",A20," ( ",I5," : ",I5," ) ")' ) "refl_ad%refl_out", &
      &"refl_ad%refl_out", ichannel1( iband ), ichannel2( iband )
    Endif

#endif

    Write( *, * )
    
  Endif


  If( opts%rt_ir%pc%addpc ) Then
      Call &
        & rttov_ad                                                                       &
        & ( errorstatus_x( iband )                                                       &
        & , chanprof_x( 1 : ichannel2( iband ) - ichannel1( iband ) + 1, iband )         &
        & , opts                                                                         &
        & , profiles( iprofile1( iband ) : iprofile2( iband ) )                          &
        & , profiles_ad_x( :, iband )                                                    &
        & , coefs                                                                        &
        & , transmission_x( iband )                                                      &
        & , transmission_ad_x( iband )                                                   &
        & , radiancedata_x( iband )                                                      &
        & , radiancedata_ad_x( iband )                                                   &
        & , calcemis       = calcemis_x( ichannel1( iband ) : ichannel2( iband ) )       &
        & , emissivity     = emissivity_x( ichannel1( iband ) : ichannel2( iband ) )     &
        & , emissivity_ad  = emissivity_ad_x( ichannel1( iband ) : ichannel2( iband ) )  &
        & , calcrefl       = calcrefl_x( ichannel1( iband ) : ichannel2( iband ) )       &
        & , reflectance    = reflectance_x( ichannel1( iband ) : ichannel2( iband ) )    &
        & , reflectance_ad = reflectance_ad_x( ichannel1( iband ) : ichannel2( iband ) ) &
        & , aer_opt_param  = aer_opt_param_x( iband )                                    &
        & , cld_opt_param  = cld_opt_param_x( iband )                                    &
        & , pccomp         = pccomp_x( iband )                                           &
        & , pccomp_ad      = pccomp_ad_x( iband )                                        &
        & , channels_rec   = channels_rec                                                &
        & )
  Else
      Call &
        & rttov_ad                                                                       &
        & ( errorstatus_x( iband )                                                       &
        & , chanprof_x( 1 : ichannel2( iband ) - ichannel1( iband ) + 1, iband )         &
        & , opts                                                                         &
        & , profiles( iprofile1( iband ) : iprofile2( iband ) )                          &
        & , profiles_ad_x( :, iband )                                                    &
        & , coefs                                                                        &
        & , transmission_x( iband )                                                      &
        & , transmission_ad_x( iband )                                                   &
        & , radiancedata_x( iband )                                                      &
        & , radiancedata_ad_x( iband )                                                   &
        & , calcemis = calcemis_x( ichannel1( iband ) : ichannel2( iband ) )             &
        & , emissivity = emissivity_x( ichannel1( iband ) : ichannel2( iband ) )         &
        & , emissivity_ad = emissivity_ad_x( ichannel1( iband ) : ichannel2( iband ) )   &
        & , calcrefl = calcrefl_x( ichannel1( iband ) : ichannel2( iband ) )             &
        & , reflectance = reflectance_x( ichannel1( iband ) : ichannel2( iband ) )       &
        & , reflectance_ad = reflectance_ad_x( ichannel1( iband ) : ichannel2( iband ) ) &
        & , aer_opt_param = aer_opt_param_x( iband )                                     &
        & , cld_opt_param = cld_opt_param_x( iband )                                     &
        & )
  EndIf

End Do
!$OMP END PARALLEL DO


 !
 ! add every contribution in output profile_k array
 !
  Do iband = 1, nbands
    Do iprofilek = iprofile1( iband ), iprofile2( iband )
!      Print *, "profiles_ad( ", iprofilek, " ) = ", &
!      & "profiles_ad( ", iprofilek, " ) + profiles_ad_x( ", &
!      & iprofilek - iprofile1( iband ) + 1, ", ", iband, " )"
      k = iprofilek - iprofile1( iband ) + 1
      Call rttov_add_prof( profiles_ad( iprofilek:iprofilek ), &
                         & profiles_ad( iprofilek:iprofilek ), &
                         & profiles_ad_x( k:k, iband ) )
    EndDo
  EndDo


!  Do iband = 1, nbands
!    Call CopyTransmission( ichannel1( iband ), ichannel2( iband ), transmission_x( iband ), transmission )
!  EndDo

  !
  ! we have to construct the global errorstatus array 
  ! from what we got from every thread
  !
  errorstatus = errorstatus_success
  If (Any(errorstatus_x( : ) /= errorstatus_success)) errorstatus = errorstatus_fatal

!  Do iband = 1, nbands
!    Do iprofilek = iprofile1( iband ), iprofile2( iband )
!      Print *, iprofilek, " <= ", iband, iprofilek - iprofile1( iband ) + 1, &
!      & errorstatus_x( iprofilek - iprofile1( iband ) + 1, iband )
!    EndDo
!  EndDo

  
  DeAllocate( chanprof_x )
  DeAllocate( errorstatus_x )
  
  If (Present(calcemis)) Then
    Nullify( calcemis_x )
  Else
    DeAllocate( calcemis_x )
  EndIf
  
  If (Present(emissivity)) Then
    Nullify( emissivity_x )
  Else
    DeAllocate( emissivity_x )
  EndIf
  
  If (Present(emissivity_ad)) Then
    Nullify( emissivity_ad_x )
  Else
    DeAllocate( emissivity_ad_x )
  EndIf

  If (Present(calcrefl)) Then
    Nullify( calcrefl_x )
  Else
    DeAllocate( calcrefl_x )
  EndIf
  
  If (Present(reflectance)) Then
    Nullify( reflectance_x )
  Else
    DeAllocate( reflectance_x )
  EndIf
  
  If (Present(reflectance_ad)) Then
    Nullify( reflectance_ad_x )
  Else
    DeAllocate( reflectance_ad_x )
  EndIf

  DeAllocate( aer_opt_param_x )
  DeAllocate( cld_opt_param_x )

  DeAllocate( transmission_x )
  DeAllocate( radiancedata_x )

  If( opts%rt_ir%pc%addpc ) DeAllocate( pccomp_x )

  DeAllocate( transmission_ad_x )
  DeAllocate( radiancedata_ad_x )
  If( opts%rt_ir%pc%addpc ) DeAllocate( pccomp_ad_x )

  Do iband = 1, nbands
    call rttov_alloc_prof( errorstatus_ad_x, nprofiles12max, profiles_ad_x( :, iband ), nlevels, opts, 0_jpim, coefs = coefs)
  EndDo
  DeAllocate( profiles_ad_x )

  DeAllocate( iprofile2 )
  DeAllocate( iprofile1 )

  If( opts%rt_ir%pc%addpc ) Then
    DeAllocate( ipc2 )
    DeAllocate( ipc1 )
    If( opts%rt_ir%pc%addradrec )Then
      DeAllocate( ichannelrec2 )
      DeAllocate( ichannelrec1 )
    EndIf
  EndIf

  DeAllocate( ichannel2 )
  DeAllocate( ichannel1 )

  Return

100 Continue
    Call rttov_errorreport( errorstatus_fatal, "Memory allocation failed", NameOfRoutine )
    Stop

Contains

  ! ancillary routines

  Subroutine AssociateRadiance( ichannel1, ichannel2, radiancedata_x, radiancedata )
    Integer(Kind=jpim), Intent(in) :: ichannel1, ichannel2
    Type(radiance_Type), Intent(in) :: radiancedata
    Type(radiance_Type), Intent(out) :: radiancedata_x

    radiancedata_x % clear &
      => radiancedata % clear( ichannel1 : ichannel2 )

    radiancedata_x % cloudy &
      => radiancedata % cloudy( ichannel1 : ichannel2 )

    radiancedata_x % total &
      => radiancedata % total( ichannel1 : ichannel2 )

    radiancedata_x % bt &
      => radiancedata % bt( ichannel1 : ichannel2 )

    radiancedata_x % bt_clear &
      => radiancedata % bt_clear( ichannel1 : ichannel2 )

    radiancedata_x % refl &
      => radiancedata % refl( ichannel1 : ichannel2 )

    radiancedata_x % refl_clear &
      => radiancedata % refl_clear( ichannel1 : ichannel2 )

    radiancedata_x % overcast &
      => radiancedata % overcast( :, ichannel1 : ichannel2 )

  End Subroutine


  Subroutine AssociateTransmission( ichannel1, ichannel2, transmission_x, transmission )
    Integer(Kind=jpim), Intent(in) :: ichannel1, ichannel2
    Type(transmission_type), Intent(in) :: transmission
    Type(transmission_type), Intent(out) :: transmission_x

    transmission_x % tau_levels &
      => transmission % tau_levels( :, ichannel1 : ichannel2 )

    transmission_x % tau_total &
      => transmission % tau_total( ichannel1 : ichannel2 )

    transmission_x % tausun_levels_path1 &
      => transmission % tausun_levels_path1( :, ichannel1 : ichannel2 )

    transmission_x % tausun_total_path1 &
      => transmission % tausun_total_path1( ichannel1 : ichannel2 )

    transmission_x % tausun_levels_path2 &
      => transmission % tausun_levels_path2( :, ichannel1 : ichannel2 )

    transmission_x % tausun_total_path2 &
      => transmission % tausun_total_path2( ichannel1 : ichannel2 )

  End Subroutine

  Subroutine AssociatePCcomp( ipc1, ipc2, pccomp_x, pccomp, ichannelrec1, ichannelrec2 )
    Integer(Kind=jpim), Intent(in) :: ipc1, ipc2
    Type(rttov_pccomp), Intent(in) :: pccomp
    Type(rttov_pccomp), Intent(out) :: pccomp_x
    Integer(Kind=jpim), Intent(in), Optional :: ichannelrec1, ichannelrec2

    pccomp_x % pcscores &
      => pccomp % pcscores( ipc1 : ipc2 )

    If( Present(ichannelrec1) ) Then
      pccomp_x % bt_pccomp &
        => pccomp % bt_pccomp( ichannelrec1 : ichannelrec2 )

      pccomp_x % total_pccomp &
        => pccomp % total_pccomp( ichannelrec1 : ichannelrec2 )
    EndIf

  End Subroutine

  Subroutine AssociateOptParam( ichannel1, ichannel2, opt_param_x, opt_param )
    Integer(Kind=jpim), Intent(in) :: ichannel1, ichannel2
    Type(rttov_opt_param), Intent(in) :: opt_param
    Type(rttov_opt_param), Intent(out) :: opt_param_x

    opt_param_x % abs &
      => opt_param % abs( ichannel1 : ichannel2, : )

    opt_param_x % sca &
      => opt_param % sca( ichannel1 : ichannel2, : )

    opt_param_x % bpr &
      => opt_param % bpr( ichannel1 : ichannel2, : )

    opt_param_x % pha &
      => opt_param % pha( ichannel1 : ichannel2, : , : )

    opt_param_x % phangle &
      => opt_param % phangle( : )

    opt_param_x % cosphangle &
      => opt_param % cosphangle( : )

    opt_param_x % iphangle &
      => opt_param % iphangle( : )

    opt_param_x % minphadiff &
      = opt_param % minphadiff

  End Subroutine

End Subroutine






